from flask import Flask
from datetime import datetime
import uuid

from cerberus.entities.entities import KsmConnection, Database
from cerberus.responses.Response import Response
from cerberus.dtos.Connection import Connection
from cerberus.mappers.ConnectionMapper import ConnectionMapper
from cerberus.exceptions.exceptions import InvalidClientConnectionException, ClientConnectionExpiredException

class ConnectionService():

    def removeConnection(token):
        if token is None:
            raise NullClientConnectionException()

        ksmConnection = KsmConnection.query.filter(KsmConnection.id == token).first()
        if ksmConnection is None:
            raise InvalidClientConnectionException()

        ksmConnection.setUpdatedAt(datetime.now())
        ksmConnection.setActive(False)
        Database.update(ksmConnection)
        return ConnectionMapper.mapToConnection(ksmConnection)

    def createConnection(client):
        token = str(uuid.uuid4())
        ksmConnection = KsmConnection(token, True, client.getId(), datetime.now(), datetime.now())
        Database.insert(ksmConnection)
        connection = Connection(ksmConnection.getId(), ksmConnection.getClientId(), ksmConnection.getActive(), ksmConnection.getCreatedAt(), ksmConnection.getUpdatedAt())
        return ConnectionMapper.mapToConnection(ksmConnection)

    def validateConnection(token):

        if token is None:
            raise NullClientConnectionException()

        ksmConnection = KsmConnection.query.filter(KsmConnection.id == token).first()

        if ksmConnection is None:
            raise InvalidClientConnectionException()

        if ksmConnection.getActive() == False:
            raise ClientConnectionExpiredException()

        ksmConnection.setUpdatedAt(datetime.now())
        Database.update(ksmConnection)
        return ConnectionMapper.mapToConnection(ksmConnection)

    def getValidConnection(token):
        if token is None:
            raise NullClientConnectionException()

        ksmConnection = KsmConnection.query.filter(KsmConnection.id == token).first()

        if ksmConnection is None:
            raise InvalidClientConnectionException()

        if ksmConnection.getActive() == False:
            raise ClientConnectionExpiredException()

        ksmConnection.setUpdatedAt(datetime.now())
        Database.update(ksmConnection)
        return ConnectionMapper.mapToConnection(ksmConnection)



from flask import Flask

class Response(dict):

    def __init__(self, headerRS, bodyRS):
        self.__headerRS = headerRS
        self.__bodyRS = bodyRS
        dict.__init__(self, headerRS=headerRS, bodyRS=bodyRS)

    def getHeaderRS(self):
        return self.__headerRS

    def getBodyRS(self):
        return self.__bodyRS



from datetime import datetime
import uuid

from cerberus.dtos.SecurityHash import SecurityHash
from cerberus.entities.entities import KsmSecurityHash, KsmSecurityHashType, Database

from cerberus.mappers.SecurityHashMapper import SecurityHashMapper
from cerberus.exceptions.exceptions import InvalidParamException, RequiredParamException, SecurityHashExpiredException, InvalidSecurityHashException

class SecurityHashService():

    def createHash(type):

        if type is None:
            raise RequiredParamException("type")

        ksmSecurityHashType = KsmSecurityHashType.query.filter(KsmSecurityHashType.id == type).first()
        if ksmSecurityHashType is None:
            raise InvalidParamException("type")

        createdAt = datetime.now()
        ksmSecurityHash = KsmSecurityHash()
        ksmSecurityHash.setToken(str(uuid.uuid4()))
        ksmSecurityHash.setUsed(False)
        ksmSecurityHash.setActive(True)
        ksmSecurityHash.setSecurityHashTypeId(type)
        ksmSecurityHash.setCreatedAt(createdAt)
        ksmSecurityHash.setUpdatedAt(createdAt)

        Database.insert(ksmSecurityHash)

        return SecurityHashMapper.mapToSecurityHash(ksmSecurityHash);

    def validateHash(token):

        if token is None:
            raise RequiredParamException("token")

        ksmSecurityHash = KsmSecurityHash.query.filter(KsmSecurityHash.token == token).first()
        if ksmSecurityHash is None:
            raise InvalidSecurityHashException()

        if ksmSecurityHash.getActive() == False or ksmSecurityHash.getUsed() == True:
            raise SecurityHashExpiredException()


        ksmSecurityHash.setUsed(True)
        Database.update(ksmSecurityHash)

        return SecurityHashMapper.mapToSecurityHash(ksmSecurityHash);



from flask import Flask

class Session(dict):

    def __init__(self, token, userId, active, authenticationType, roleId, connection, createdAt, updatedAt):
        self.__token = token
        self.__userId = userId
        self.__active = active
        self.__authenticationType = authenticationType
        self.__roleId = roleId
        self.__connection = connection
        self.__updatedAt = updatedAt
        self.__createdAt = createdAt
        dict.__init__(self, token=token, userId=userId, active = active, authenticationType=authenticationType, roleId= roleId, connection = connection, createdAt=str(createdAt),updatedAt=str(updatedAt))

    def getToken(self):
        return self.__token

    def getUserId(self):
        return self.__userId

    def getActive(self):
        return self.__active

    def getAuthenticationType(self):
        return self.__authenticationType

    def getRoleId(self):
        return self.__roleId

    def getConnection(self):
        return self.__connection

    def setConnection(self, connection):
        self.__connection = connection

    def getUpdatedAt(self):
        return self.__updatedAt

    def getCreatedAt(self):
        return self.__createdAt



from flask import Flask

class HeaderRQ:

    def __init__(self, token, languageCode, xTimestamp):
        self.__token = token
        self.__languageCode = languageCode
        self.__xTimestamp = xTimestamp

    def getToken(self):
        return self.__token

    def getLanguageCode(self):
        return self.__languageCode

    def getXTimestamp(self):
        return self.__xTimestamp



from flask import Flask

from cerberus.responses.HeaderRS import HeaderRS

class HeaderMapper():

    def mapToHeader(session):
        header = HeaderRS(session.getToken(),session.getCreatedAt(),session.getUpdatedAt());
        return header;



from flask import Flask

from cerberus.dtos.Error import Error

class BodyRS(dict):

    def __init__(self, success, error= None):
        self.__success = success
        self.__error = error
        dict.__init__(self, success=success, error=error)


    def getSuccess(self):
        return self.__success

    def getError(self):
        return self.__error

    def setSuccess(self, success):
        self.__success = success

    def setError(self, error):
        self.__error = error



from datetime import datetime, timedelta, timezone
import time



class DateUtils():

    def strToFullDate(strDate):
        return datetime.strptime(strDate,'%Y-%m-%d %H:%M:%S')

    def strToDate(strDate):
        return datetime.strptime(strDate,'%Y-%m-%d')

    def datToShortFormat(date):
        date = datetime(year=date.year,
                      month=date.month,
                      day=date.day,
                      hour=0,
                      minute=0,
                      second=0)
        return date

    def dateToFirstHour(date):
        utc_offset = time.timezone/3600
        date = datetime(year=date.year,
                      month=date.month,
                      day=date.day,
                      hour=0,
                      minute=0,
                      second=0)
        date = date + timedelta(hours=utc_offset)
        return date

    def dateToFinalHour(date):
        utc_offset = time.timezone/3600
        date = datetime(year=date.year,
                      month=date.month,
                      day=date.day,
                      hour=23,
                      minute=59,
                      second=59)

        date = date + timedelta(hours=utc_offset)
        return date



from cerberus.responses.BodyRS import BodyRS

class CreateUserAccessTokenRS(BodyRS):

    def __init__(self, success, error=None):
        BodyRS.__init__(self, success, error)

    def getSecurityHash(self):
        return self.__securityHash

    def setSecurityHash(self, securityHash):
        self.__securityHash = securityHash
        self.update(securityHash=securityHash)



from flask import Flask

from cerberus.requests.BodyRQ import BodyRQ

class CreateUserAccessTokenRQ(BodyRQ):

    def __init__(self, email):
        self.__email = email

    def getEmail(self):
        return self.__email



from flask import Flask

class Connection(dict):

    __token = ''
    __clientId = 0
    __active = False
    __createdAt = ''
    __updatedAt = ''

    def __init__(self, token, clientId, active, createdAt, updatedAt):
        self.__token = token
        self.__clientId = clientId
        self.__active = active
        self.__createdAt = createdAt
        self.__updatedAt = updatedAt
        dict.__init__(self, token=token, clientId=clientId, active=active, createdAt=str(createdAt),updatedAt=str(updatedAt))

    def getToken(self):
        return self.__token

    def getClientId(self):
        return self.__clientId

    def getActive(self):
        return self.__active

    def getUpdatedAt(self):
        return self.__updatedAt

    def getCreatedAt(self):
        return self.__createdAt



from flask import Flask

class AbstractException(Exception):

    __code=0;
    __message='';

    def __init__(self, code, message):
        Exception.__init__(self,message)
        self.__code = code;
        self.__message = message;

    def getCode(self):
        return self.__code

    def getMessage(self):
        return self.__message

    def to_dict(self):
        rv = dict()
        rv['code'] = self.__code
        rv['message'] = self.__message
        return rv

class InvalidClientCredentialsException(AbstractException):
    def __init__(self,*args,**kwargs):
        AbstractException.__init__(self,2001, "The client credentials are invalid.")

class InvalidClientConnectionException(AbstractException):
    def __init__(self,*args,**kwargs):
        AbstractException.__init__(self,2003, "The client information is invalid.")

class NullParamsException(AbstractException):
    def __init__(self,*args,**kwargs):
        AbstractException.__init__(self,1010, "The params are required.")

class RequiredParamException(AbstractException):
    def __init__(self,*args,**kwargs):
        AbstractException.__init__(self,1011, "The %s param is required." % args[0])

class InternalErrorException(AbstractException):
    def __init__(self,*args,**kwargs):
        AbstractException.__init__(self,5000, "Internal Error Exception. %s" % args[0])

class InvalidUserCredentialsException(AbstractException):
    def __init__(self,*args,**kwargs):
        AbstractException.__init__(self,2007, "The user credentials are invalid.")

class NotFoundUserException(AbstractException):
    def __init__(self,*args,**kwargs):
        AbstractException.__init__(self,1021, "User Not found exception.")

class NullClientConnectionException(AbstractException):
    def __init__(self,*args,**kwargs):
        AbstractException.__init__(self,2002, "Client connection not found.")

class ClientConnectionExpiredException(AbstractException):
    def __init__(self,*args,**kwargs):
        AbstractException.__init__(self,2004, "The client connection has expired.")

class UserCredentialsExpiredException(AbstractException):
    def __init__(self,*args,**kwargs):
        AbstractException.__init__(self,2018, "The user credentials has expired.")

class MaxUserSessionException(AbstractException):
    def __init__(self,*args,**kwargs):
        AbstractException.__init__(self,2010, "Too many user sessions.")

class InvalidUserSessionException(AbstractException):
    def __init__(self,*args,**kwargs):
        AbstractException.__init__(self,2016, "The invalid user's session exception.")

class InvalidParamException(AbstractException):
    def __init__(self,*args,**kwargs):
        AbstractException.__init__(self,1000, "The param is invalid: %s " % args[0])

class UserAlreadyExistException(AbstractException):
    def __init__(self,*args,**kwargs):
        AbstractException.__init__(self,5010, "The user already exists.")

class NotFoundRoleException(AbstractException):
    def __init__(self,*args,**kwargs):
        AbstractException.__init__(self,1031, "Not found role exception.")

class InvalidSecurityHashException(AbstractException):
    def __init__(self,*args,**kwargs):
        AbstractException.__init__(self,1200, "Security hash is invalid.")

class SecurityHashExpiredException(AbstractException):
    def __init__(self,*args,**kwargs):
        AbstractException.__init__(self,1204, "Security hash has expired.")

class ClientServiceNotAllowedException(AbstractException):
    def __init__(self,*args,**kwargs):
        AbstractException.__init__(self,2015, "The client not allowed %s service." % args[0])

class ServiceNotAllowedException(AbstractException):
    def __init__(self,*args,**kwargs):
        AbstractException.__init__(self,2015, "The service not allowed %s." % args[0])

class UserServiceNotAllowedException(AbstractException):
    def __init__(self,*args,**kwargs):
        AbstractException.__init__(self,2018, "User not allowed to execute %s service." % args[0])



from cerberus.responses.BodyRS import BodyRS

class LoginClientRS(BodyRS):

    __connection = ''

    def __init__(self, success, error=None):
        BodyRS.__init__(self, success, error)

    def getConnection(self):
        return self.__connection

    def setConnection(self, connection):
        self.__connection = connection
        self.update(connection=connection)



from flask import Flask

class Request:

    def __init__(self, headerRQ, bodyRQ=None):
        self.__headerRQ = headerRQ
        self.__bodyRQ = bodyRQ

    def getHeaderRQ(self):
        return self.__headerRQ

    def getBodyRQ(self):
        return self.__bodyRQ



from flask import Flask

class HeaderRS(dict):

    __token = ''
    __updatedAt = ''
    __createdAt = ''

    def __init__(self, token=None, createdAt=None, updatedAt=None):
        self.__token = token
        self.__createdAt = createdAt
        self.__updatedAt = updatedAt
        dict.__init__(self, token=token, createdAt=str(createdAt), updatedAt=str(updatedAt))

    def getToken(self):
        return self.__token

    def getUpdatedAt(self):
        return self.__updatedAt

    def getCreatedAt(self):
        return self.__createdAt



from cerberus.management.SessionManagerAbstractService import SessionManagerAbstractService
from abc import abstractmethod

class ConnectionExecuteService(SessionManagerAbstractService):

    def __init__(self, request):
        SessionManagerAbstractService.__init__(self, request)

    def beforeExecute(self, connection):
        pass

    @abstractmethod
    def execute(self, connection):
        pass


    def afterExecute(self, connection):
        pass



from flask import Flask
from app import db

class KsmConnection(db.Model):
    __tablename__ = 'ksm_connection'
    __bind_key__ = 'intranet_inovafit'
    __table_args__ = {'schema': 'intranet_inovafit'}

    id = db.Column(db.String(60), primary_key=True)
    active = db.Column(db.Boolean, nullable=False)
    client_id = db.Column(db.Integer, nullable=False)
    created_at = db.Column(db.DateTime, nullable=False)
    updated_at = db.Column(db.DateTime, nullable=False)
    deleted_at = db.Column(db.DateTime, nullable=False)

    def __init__(self, id, active, client_id,created_at, updated_at, deleted_at=None):
       self.id = id
       self.active = active
       self.client_id = client_id
       self.created_at = created_at
       self.updated_at = updated_at
       self.deleted_at = deleted_at

    def getId(self):
       return self.id

    def getActive(self):
       return self.active

    def setActive(self, active):
       self.active = active

    def getClientId(self):
       return self.client_id

    def getCreatedAt(self):
       return self.created_at

    def getUpdatedAt(self):
       return self.updated_at

    def setUpdatedAt(self, updated_at):
       self.updated_at = updated_at

    def getDeletedAt(self):
       return self.deleted_at

class KsmClient(db.Model):
    __tablename__ = 'ksm_client'
    __bind_key__ = 'intranet_inovafit'
    id = db.Column(db.Integer, primary_key=True)
    active = db.Column(db.Boolean, nullable=False)
    username = db.Column(db.String(100), nullable=False)
    password = db.Column(db.String(150), nullable=False)

    def getId(self):
       return self.id

    def getUsername(self):
       return self.username

    def getPassword(self):
       return self.password

    def getActive(self):
      return self.active

class KsmSession(db.Model):
    __tablename__ = 'ksm_session'
    __bind_key__ = 'intranet_inovafit'
    id = db.Column(db.String(60), primary_key=True)
    active = db.Column(db.Boolean, nullable=False)
    connection_id = db.Column(db.Integer, nullable=False)
    user_id = db.Column(db.Integer, nullable=False)
    authentication_type_id = db.Column(db.Integer, nullable=False)
    created_at = db.Column(db.DateTime, nullable=False)
    updated_at = db.Column(db.DateTime, nullable=False)
    deleted_at = db.Column(db.DateTime, nullable=False)

    def __init__(self, id, active, connection_id, user_id, authentication_type_id, created_at, updated_at, deleted_at=None):
       self.id = id
       self.active = active
       self.connection_id = connection_id
       self.user_id = user_id
       self.authentication_type_id = authentication_type_id
       self.created_at = created_at
       self.updated_at = updated_at
       self.deleted_at = deleted_at

    def getId(self):
       return self.id

    def getActive(self):
       return self.active

    def getConnectionId(self):
       return self.connection_id

    def getUserId(self):
       return self.user_id

    def getAuthenticationTypeId(self):
       return self.authentication_type_id

    def getCreatedAt(self):
       return self.created_at

    def getUpdatedAt(self):
       return self.updated_at

    def setUpdatedAt(self, updated_at):
       self.updated_at = updated_at

    def getDeletedAt(self):
       return self.deleted_at

class KsmUserAuthenticationType(db.Model):
    __tablename__ = 'ksm_user_authentication_type'
    __bind_key__ = 'intranet_inovafit'
    id = db.Column(db.Integer, primary_key=True)
    username = db.Column(db.String(100), nullable=False)
    token = db.Column(db.String(1500), nullable=False)
    authentication_type_id = db.Column(db.Integer, nullable=False)
    user_id = db.Column(db.Integer, nullable=False)
    created_at = db.Column(db.DateTime, nullable=False)
    updated_at = db.Column(db.DateTime, nullable=False)
    deleted_at = db.Column(db.DateTime, nullable=False)

    def getId(self):
       return self.id

    def getUserName(self):
       return self.username

    def getToken(self):
       return self.token

    def getAuthenticationTypeId(self):
       return self.authentication_type_id

    def getUserId(self):
       return self.user_id

    def getCreatedAt(self):
       return self.created_at

    def getUpdatedAt(self):
       return self.updated_at

    def setId(self, id):
       self.id = id

    def setUserName(self, username):
       self.username = username

    def setToken(self, token):
       self.token = token

    def setAuthenticationTypeId(self, authentication_type_id):
       self.authentication_type_id = authentication_type_id

    def setUserId(self, user_id):
       self.user_id = user_id

    def setCreatedAt(self, created_at):
       self.created_at = created_at

    def setUpdatedAt(self, updated_at):
       self.updated_at = updated_at

    def getDeletedAt(self):
       return self.deleted_at

class KsmRole(db.Model):
    __tablename__ = 'ksm_role'
    __bind_key__ = 'intranet_inovafit'
    id = db.Column(db.Integer, primary_key=True)
    description = db.Column(db.String(300), nullable=False)
    created_at = db.Column(db.DateTime, nullable=False)
    updated_at = db.Column(db.DateTime, nullable=False)
    deleted_at = db.Column(db.DateTime, nullable=False)

    def getId(self):
       return self.id

    def getDescription(self):
       return self.description

    def getCreatedAt(self):
       return self.created_at

    def getUpdatedAt(self):
       return self.updated_at

    def getDeletedAt(self):
       return self.deleted_at

class KsmUser(db.Model):
    __tablename__ = 'ksm_user'
    __bind_key__ = 'intranet_inovafit'
    id = db.Column(db.String(60), primary_key=True)
    created_at = db.Column(db.DateTime, nullable=False)
    updated_at = db.Column(db.DateTime, nullable=False)
    deleted_at = db.Column(db.DateTime, nullable=False)

    def setId(self, id):
       self.id = id

    def getId(self):
       return self.id

    def setCreatedAt(self, created_at):
       self.created_at = created_at

    def setUpdatedAt(self, updated_at):
       self.updated_at = updated_at

    def getCreatedAt(self):
       return self.created_at

    def getUpdatedAt(self):
       return self.updated_at

    def getDeletedAt(self):
       return self.deleted_at

class KsmRoleUser(db.Model):
    __tablename__ = 'ksm_role_user'
    __bind_key__ = 'intranet_inovafit'
    id = db.Column(db.Integer, primary_key=True)
    role_id = db.Column(db.Integer, nullable=False)
    user_id = db.Column(db.Integer, nullable=False)
    created_at = db.Column(db.DateTime, nullable=False)
    updated_at = db.Column(db.DateTime, nullable=False)
    deleted_at = db.Column(db.DateTime, nullable=False)

    def getId(self):
       return self.id

    def getRoleId(self):
       return self.role_id

    def getUserId(self):
       return self.user_id

    def getCreatedAt(self):
       return self.created_at

    def getUpdatedAt(self):
       return self.updated_at

    def getDeletedAt(self):
       return self.deleted_at

    def setRoleId(self, role_id):
       self.role_id = role_id

    def setUserId(self, user_id):
       self.user_id = user_id

    def setCreatedAt(self, created_at):
       self.created_at = created_at

    def setUpdatedAt(self, updated_at):
       self.updated_at = updated_at

class KsmRoleService(db.Model):
    __tablename__ = 'ksm_role_service'
    __bind_key__ = 'intranet_inovafit'
    id = db.Column(db.Integer, primary_key=True)
    role_id = db.Column(db.Integer, nullable=False)
    service_id = db.Column(db.Integer, nullable=False)
    created_at = db.Column(db.DateTime, nullable=False)
    updated_at = db.Column(db.DateTime, nullable=False)
    deleted_at = db.Column(db.DateTime, nullable=False)

    def getId(self):
       return self.id

    def getRoleId(self):
       return self.role_id

    def getServiceId(self):
       return self.service_id

    def getCreatedAt(self):
       return self.created_at

    def getUpdatedAt(self):
       return self.updated_at

    def getDeletedAt(self):
       return self.deleted_at

    def setRoleId(self, role_id):
       self.role_id = role_id

    def setServiceId(self, user_id):
       self.service_id = user_id

    def setCreatedAt(self, created_at):
       self.created_at = created_at

    def setUpdatedAt(self, updated_at):
       self.updated_at = updated_at

class KsmSecurityHashType(db.Model):
    __tablename__ = 'ksm_security_hash_type'
    __bind_key__ = 'intranet_inovafit'
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(255), nullable=False)
    created_at = db.Column(db.DateTime, nullable=False)

    def getId(self):
       return self.id

    def getName(self):
       return self.name

    def getCreatedAt(self):
       return self.created_at

class KsmSecurityHash(db.Model):

    __tablename__ = 'ksm_security_hash'
    __bind_key__ = 'intranet_inovafit'
    id = db.Column(db.Integer, primary_key=True)
    token = db.Column(db.String(255), nullable=False)
    used = db.Column(db.Boolean, nullable=False)
    active = db.Column(db.Boolean, nullable=False)
    security_hash_type_id = db.Column(db.Integer, nullable=False)
    created_at = db.Column(db.DateTime, nullable=False)
    updated_at = db.Column(db.DateTime, nullable=False)
    deleted_at = db.Column(db.DateTime, nullable=False)

    def getId(self):
       return self.id

    def getToken(self):
       return self.token

    def getUsed(self):
       return self.used

    def getActive(self):
       return self.active

    def getSecurityHashTypeId(self):
       return self.security_hash_type_id

    def getCreatedAt(self):
       return self.created_at

    def getUpdatedAt(self):
       return self.updated_at

    def getDeletedAt(self):
       return self.deleted_at

    def setToken(self, token):
       self.token = token

    def setSecurityHashTypeId(self, security_hash_type_id):
       self.security_hash_type_id = security_hash_type_id

    def setUsed(self, used):
       self.used = used

    def setActive(self, active):
       self.active = active

    def setCreatedAt(self, created_at):
       self.created_at = created_at

    def setUpdatedAt(self, updated_at):
       self.updated_at = updated_at

class KsmClientService(db.Model):

    __tablename__ = 'ksm_client_service'
    __bind_key__ = 'intranet_inovafit'
    id = db.Column(db.Integer, primary_key=True)
    client_id = db.Column(db.Integer, nullable=False)
    service_id = db.Column(db.Integer, nullable=False)
    created_at = db.Column(db.DateTime, nullable=False)
    updated_at = db.Column(db.DateTime, nullable=False)
    deleted_at = db.Column(db.DateTime, nullable=False)

    def getId(self):
       return self.id

    def getClientId(self):
       return self.client_id

    def getServiceId(self):
       return self.service_id

    def getCreatedAt(self):
       return self.created_at

    def getUpdatedAt(self):
       return self.updated_at

    def getDeletedAt(self):
       return self.deleted_at

class KsmUserService(db.Model):

    __tablename__ = 'ksm_user_service'
    __bind_key__ = 'intranet_inovafit'
    id = db.Column(db.Integer, primary_key=True)
    user_id = db.Column(db.Integer, nullable=False)
    service_id = db.Column(db.Integer, nullable=False)
    created_at = db.Column(db.DateTime, nullable=False)
    updated_at = db.Column(db.DateTime, nullable=False)
    deleted_at = db.Column(db.DateTime, nullable=False)

    def getId(self):
       return self.id

    def getUserId(self):
       return self.user_id

    def getServiceId(self):
       return self.service_id

    def getCreatedAt(self):
       return self.created_at

    def getUpdatedAt(self):
       return self.updated_at

    def getDeletedAt(self):
       return self.deleted_at

class KsmService(db.Model):

    __tablename__ = 'ksm_service'
    __bind_key__ = 'intranet_inovafit'
    id = db.Column(db.Integer, primary_key=True)
    name = db.Column(db.String(100), nullable=False)
    description = db.Column(db.String(100), nullable=False)
    created_at = db.Column(db.DateTime, nullable=False)
    updated_at = db.Column(db.DateTime, nullable=False)
    deleted_at = db.Column(db.DateTime, nullable=False)

    def getId(self):
       return self.id

    def getName(self):
       return self.name

    def getDescription(self):
       return self.description

    def getCreatedAt(self):
       return self.created_at

    def getUpdatedAt(self):
       return self.updated_at

    def getDeletedAt(self):
       return self.deleted_at


class Database():
    def update(obj):
        db.session.commit()

    def insert(obj):
        db.session.add(obj)
        db.session.commit()



from cerberus.responses.BodyRS import BodyRS

class CreateUserRS(BodyRS):

    def __init__(self, success, error=None):
        BodyRS.__init__(self, success, error)

    def getUser(self):
        return self.__user

    def setUser(self, user):
        self.__user = user
        self.update(user=user)




from cerberus.dtos.SecurityHash import SecurityHash

class SecurityHashMapper():

    def mapToSecurityHash(ksmSecurityHash):
        securityHash = SecurityHash(ksmSecurityHash.getToken(), ksmSecurityHash.getSecurityHashTypeId(), ksmSecurityHash.getUsed(), ksmSecurityHash.getActive())
        return securityHash



from cerberus.dtos.Session import Session
from cerberus.entities.entities import KsmRoleUser

class SessionMapper():
    def mapToSession(ksmSession, connection):
        roleUser = KsmRoleUser.query.filter(KsmRoleUser.user_id == ksmSession.getUserId()).first()
        return Session(ksmSession.getId(), ksmSession.getUserId(), ksmSession.getActive(), ksmSession.getAuthenticationTypeId(), roleUser.getRoleId(), connection, ksmSession.getCreatedAt(), ksmSession.getUpdatedAt())




class SecurityHash(dict):
    AUTH_TOKEN = 1
    CREATE_USER_TOKEN = 2

    def __init__(self, token, typeId, used, active):
        self.__token = token
        self.__typeId = typeId
        self.__used = used
        self.__active = active
        dict.__init__(self, token=token, typeId=typeId, used = used, active=active)

    def getToken(self):
        return self.__token

    def getTypeId(self):
        return self.__typeId

    def getUsed(self):
        return self.__used

    def getActive(self):
        return self.__active



from cerberus.exceptions.exceptions import ServiceNotAllowedException, ClientServiceNotAllowedException, UserServiceNotAllowedException, NullClientConnectionException, InvalidClientConnectionException, ClientConnectionExpiredException
from cerberus.entities.entities import KsmRoleService, KsmClientService, KsmUserService, KsmRoleUser, KsmService
from cerberus.management.ConnectionExecuteService import ConnectionExecuteService
from cerberus.management.SessionExecuteService import SessionExecuteService
from cerberus.services.ConnectionService import ConnectionService
from cerberus.services.SessionService import SessionService

class AccessControlList():

    def validateExecuteService(service):
        if isinstance(service, ConnectionExecuteService):
            return AccessControlList.validateConnectionExecuteService(service)
        elif isinstance(service, SessionExecuteService):
            return AccessControlList.validateSessionExecuteService(service)

    def validateConnectionExecuteService(service):
        connection = ConnectionService.getValidConnection(service.getRequest().getHeaderRQ().getToken())

        ksmService = KsmService.query.filter(KsmService.name == service.__class__.__name__).first()
        if ksmService is None:
            raise ServiceNotAllowedException(service.__class__.__name__)

        ksmClientService = KsmClientService.query.filter(KsmClientService.client_id == connection.getClientId()).filter(KsmClientService.service_id == ksmService.getId()).first()
        if ksmClientService is None:
            print("ksmClientService is None: " + service.__class__.__name__)
            raise ClientServiceNotAllowedException(service.__class__.__name__)

        return connection

    def validateSessionExecuteService(service):

        session = SessionService.getValidSession(service.getRequest().getHeaderRQ().getToken())
        connectionId = SessionService.getConnectionIdBySession(service.getRequest().getHeaderRQ().getToken())
        connection = ConnectionService.getValidConnection(connectionId)
        session.setConnection(connection)

        ksmService = KsmService.query.filter(KsmService.name == service.__class__.__name__).first()
        if ksmService is None:
            raise ServiceNotAllowedException(service.__class__.__name__)

        ksmClientService = KsmClientService.query.filter(KsmClientService.client_id == connection.getClientId()).filter(KsmClientService.service_id == ksmService.getId()).first()
        if ksmClientService is None:
            raise ClientServiceNotAllowedException(service.__class__.__name__)


        rolesUser = KsmRoleUser.query.filter(KsmRoleUser.user_id == session.getUserId()).all()
        allowedExecution = False
        if not rolesUser is None:
            for roleUser in rolesUser:
                ksmRoleService = KsmRoleService.query.filter(KsmRoleService.role_id == roleUser.getRoleId()).filter(KsmRoleService.service_id == ksmService.getId()).first()
                if not ksmRoleService is None:
                    allowedExecution = True

        if allowedExecution == True:
            return session

        ksmUserService = KsmUserService.query.filter(KsmUserService.user_id == session.getUserId()).filter(KsmUserService.service_id == ksmClientService.getServiceId()).first()
        if ksmUserService is None:
            raise UserServiceNotAllowedException(service.__class__.__name__)

        return session



from flask import Flask

from cerberus.requests.BodyRQ import BodyRQ

class CreateUserRQ(BodyRQ):

    def __init__(self, securityToken, username, token, roleId, authenticationTypeId):
        self.__securityToken = securityToken
        self.__username = username
        self.__token = token
        self.__roleId = roleId
        self.__authenticationTypeId = authenticationTypeId

    def getSecurityToken(self):
        return self.__securityToken

    def getUserName(self):
        return self.__username

    def getToken(self):
        return self.__token

    def getRoleId(self):
        return self.__roleId

    def getAuthenticationTypeId(self):
        return self.__authenticationTypeId



class User(dict):

    def __init__(self, id, createdAt, updatedAt):
        self.__id = id
        self.__updatedAt = updatedAt
        self.__createdAt = createdAt
        dict.__init__(self, id=id, createdAt=str(createdAt), updatedAt=str(updatedAt))

    def getId(self):
        return self.__id

    def getUpdatedAt(self):
        return self.__updatedAt

    def getCreatedAt(self):
        return self.__createdAt



from flask import Flask

class Role:

    __id = 0
    __description = ''

    def __init__(self, id, description):
        self.__id = id
        self.__description = description

    def getId(self):
        return self.__id

    def getDescription(self):
        return self.__description



import firebase_admin
from firebase_admin import auth, credentials
from cerberus.exceptions.exceptions import InvalidUserCredentialsException, NotFoundUserException

class FirebaseService():

    def __init__(self):
        cred = credentials.Certificate("/Users/cesarocotitla/Projects/intranet_inovafit_web/key/intranet-inovafit-firebase-adminsdk-dbrq9-92b838f9a9.json")
        if (not len(firebase_admin._apps)):
            default_app = firebase_admin.initialize_app(cred, {'databaseURL': 'https://intranet-inovafit.firebaseio.com'})

    def getDecoded_token(self, tokenId):
        try:
            decoded_token = auth.verify_id_token(tokenId)
        except Exception as error:
             raise InvalidUserCredentialsException()

        return decoded_token



from flask import Flask
import uuid

from datetime import datetime
from werkzeug.security import check_password_hash, generate_password_hash
from cerberus.entities.entities import KsmConnection, KsmSession, KsmUserAuthenticationType, Database
from cerberus.dtos.AuthenticationType import AuthenticationType
from cerberus.mappers.SessionMapper import SessionMapper
from cerberus.exceptions.exceptions import InvalidUserSessionException, InvalidUserCredentialsException, NotFoundUserException
from cerberus.services.SecurityHashService import SecurityHashService
from cerberus.services.FirebaseService import FirebaseService

class SessionService():

    def createSessionByLocalAuth(connection, username, password):

        if username is None:
            raise InvalidUserCredentialsException()

        if password is None:
            raise InvalidUserCredentialsException()

        kuat = KsmUserAuthenticationType.query.filter(KsmUserAuthenticationType.username == username).filter(KsmUserAuthenticationType.authentication_type_id == AuthenticationType.LOCAL).first()

        if kuat is None:
            raise NotFoundUserException()


        if not check_password_hash(kuat.getToken(), kuat.getUserId() + password):
            raise InvalidUserCredentialsException()

        return SessionService.createSession(connection, kuat)

    def createSessionByGoogleAuth(connection, uid, tokenId):

        if uid is None:
            raise InvalidUserCredentialsException()

        if tokenId is None:
            raise InvalidUserCredentialsException()

        decoded_token = FirebaseService().getDecoded_token(tokenId)

        res_uid = decoded_token['uid']
        if res_uid != uid:
            raise InvalidUserCredentialsException()

        kuat = KsmUserAuthenticationType.query.filter(KsmUserAuthenticationType.username == uid).filter(KsmUserAuthenticationType.authentication_type_id == AuthenticationType.GOOGLE).first()
        if kuat is None:
            user_email = decoded_token['email']
            kuat = KsmUserAuthenticationType.query.filter(KsmUserAuthenticationType.username == user_email).first()
            if kuat is None:
                raise InvalidUserCredentialsException()
            else:
                createdAt = datetime.now()
                ksmUserAuthenticationType = KsmUserAuthenticationType()
                ksmUserAuthenticationType.setAuthenticationTypeId(AuthenticationType.GOOGLE)
                ksmUserAuthenticationType.setUserId(kuat.getUserId())
                ksmUserAuthenticationType.setUserName(uid)
                ksmUserAuthenticationType.setCreatedAt(createdAt)
                ksmUserAuthenticationType.setUpdatedAt(createdAt)
                ksmUserAuthenticationType.setToken(uid)
                Database.insert(ksmUserAuthenticationType)
                return SessionService.createSession(connection, ksmUserAuthenticationType)

        return SessionService.createSession(connection, kuat)


    #def createSessionByFacebookAuth(self, connection, token):

    def createSessionByTokenAuth(connection, username, token):

        if username is None:
            raise InvalidUserCredentialsException()

        if token is None:
            raise InvalidUserCredentialsException()

        kuat = KsmUserAuthenticationType.query.filter(KsmUserAuthenticationType.username == username).filter(KsmUserAuthenticationType.authentication_type_id == AuthenticationType.TOKEN).first()

        if kuat is None:
            raise NotFoundUserException()

        SecurityHashService.validateHash(token)

        if not check_password_hash(kuat.getToken(), token):
            raise InvalidUserCredentialsException()

        return SessionService.createSession(connection, kuat)

    def createSession(connection, kuat):

        if kuat is None:
            return

        date = datetime.now()
        ksmSession = KsmSession(str(uuid.uuid4()), True, connection.getToken(), kuat.getUserId(), kuat.getAuthenticationTypeId(), date, date)
        Database.insert(ksmSession)
        return SessionMapper.mapToSession(ksmSession, connection)

    def getValidSession(token):

        if token is None:
            raise NullClientSessionException()

        ksmSession = KsmSession.query.filter(KsmSession.id == token).first()
        if ksmSession is None:
            raise InvalidUserSessionException()

        if ksmSession.getActive() == False:
            raise ClientSessionExpiredException()

        date = datetime.now()
        ksmSession.setUpdatedAt(date)
        Database.update(ksmSession)

        return SessionMapper.mapToSession(ksmSession, None)


    def getConnectionIdBySession(token):

        if token is None:
            raise NullClientSessionException()

        ksmSession = KsmSession.query.filter(KsmSession.id == token).first()
        if ksmSession is None:
            raise InvalidUserSessionException()

        return ksmSession.getConnectionId()



from cerberus.exceptions.exceptions import InvalidUserSessionException, ServiceNotAllowedException, ClientServiceNotAllowedException, UserServiceNotAllowedException, NullClientConnectionException, InvalidClientConnectionException, ClientConnectionExpiredException
from cerberus.responses.Response import Response
from cerberus.responses.BodyRS import BodyRS
from cerberus.requests.Request import Request
from cerberus.responses.HeaderRS import HeaderRS
from cerberus.acl.AccessControlList import AccessControlList
from cerberus.dtos.Error import Error
class SessionManager():

    def execute(self, service):
        headerRS = HeaderRS(service.getRequest().getHeaderRQ().getToken())
        try:
            cso = AccessControlList.validateExecuteService(service)
            service.beforeExecute(cso)
            bodyRS = service.execute(cso)
            service.afterExecute(cso)
        except (InvalidUserSessionException, ServiceNotAllowedException, ClientServiceNotAllowedException, UserServiceNotAllowedException, NullClientConnectionException, InvalidClientConnectionException, ClientConnectionExpiredException) as e:
            error = Error(e.getCode(), e.getMessage())
            bodyRS = BodyRS(False, error)

        response = Response(headerRS, bodyRS)
        return response



from cerberus.responses.BodyRS import BodyRS

class SessionRS(BodyRS):

    __session = ''

    def __init__(self, success, error=None):
        BodyRS.__init__(self, success, error)

    def getSession(self):
        return self.__session

    def setSession(self, session):
        self.__session = session
        self.update(session=session)



from setuptools import setup, find_packages
 
setup(name='kea_cerberus',
      version='0.3',
      url='http://kea.mx/',
      packages=find_packages(exclude=['tests']),	
      license='MIT',
      author='Cesar Ocotitla',
      author_email='cesar@kea.mx',
      description='Control client, users and service execution of your projects in python',
      zip_safe=False)



from flask import Flask

from werkzeug.security import generate_password_hash, check_password_hash
from cerberus.requests.Request import Request
from cerberus.requests.LoginUserRQ import LoginUserRQ
from cerberus.requests.CreateUserRQ import CreateUserRQ
from cerberus.responses.Response import Response
from cerberus.responses.BodyRS import BodyRS
from cerberus.responses.LoginClientRS import LoginClientRS
from cerberus.responses.SessionRS import SessionRS
from cerberus.responses.HeaderRS import HeaderRS
from cerberus.responses.CreateUserRS import CreateUserRS
from cerberus.responses.CreateUserAccessTokenRS import CreateUserAccessTokenRS
from cerberus.entities.entities import KsmClient, KsmConnection
from cerberus.dtos.Connection import Connection
from cerberus.dtos.AuthenticationType import AuthenticationType
from cerberus.dtos.Session import Session
from cerberus.dtos.Error import Error
from cerberus.dtos.User import User
from cerberus.dtos.SecurityHash import SecurityHash
from cerberus.services.ConnectionService import ConnectionService
from cerberus.services.SessionService import SessionService
from cerberus.services.UserService import UserService
from cerberus.mappers.HeaderMapper import HeaderMapper
from cerberus.services.SecurityHashService import SecurityHashService

from cerberus.exceptions.exceptions import InvalidClientCredentialsException, InvalidUserCredentialsException, NotFoundUserException
from cerberus.exceptions.exceptions import InvalidClientConnectionException, ClientConnectionExpiredException, RequiredParamException, NullClientConnectionException
from cerberus.exceptions.exceptions import UserCredentialsExpiredException, InternalErrorException, MaxUserSessionException, InvalidUserSessionException, InvalidParamException
from cerberus.exceptions.exceptions import NotFoundRoleException, UserAlreadyExistException, SecurityHashExpiredException, InvalidSecurityHashException


class Authenticate():

    #Method to create client connection
    def login(username, password):
        bodyRS = LoginClientRS(True)
        headerRS = HeaderRS()

        try:
            if username is None:
                raise RequiredParamException("username")

            if password is None:
                raise RequiredParamException("password")

            #print("hash")
            #print(generate_password_hash(password))

            client = KsmClient.query.filter(KsmClient.username == username).filter(KsmClient.active == True).first()

            if client is None:
                raise InvalidClientCredentialsException()

            if not check_password_hash(client.getPassword(), password):
                raise InvalidClientCredentialsException()

            connection = ConnectionService.createConnection(client)
            headerRS = HeaderRS(connection.getToken(), connection.getCreatedAt(), connection.getUpdatedAt())
            bodyRS.setConnection(connection)


        except (RequiredParamException, InvalidClientCredentialsException) as e:
            error = Error(e.getCode(), e.getMessage())
            bodyRS = LoginClientRS(False, error)

        response = Response(headerRS, bodyRS)
        return response

    def logout(request):
        bodyRS = BodyRS(True)
        headerRS = HeaderRS()

        try:
            connection = ConnectionService.removeConnection(request.getHeaderRQ().getToken())
            headerRS = HeaderRS(connection.getToken(), connection.getCreatedAt(), connection.getUpdatedAt())
        except (NullClientConnectionException, InvalidClientCredentialsException) as e:
            error = Error(e.getCode(), e.getMessage())
            bodyRS = BodyRS(False, error)

        response = Response(headerRS, bodyRS)
        return response;


        return response

    def validate(request):
        bodyRS = BodyRS(True)
        headerRS = HeaderRS()

        try:
            connection = ConnectionService.validateConnection(request.getHeaderRQ().getToken())
            headerRS = HeaderRS(connection.getToken(), connection.getCreatedAt(), connection.getUpdatedAt())
        except (NullClientConnectionException, InvalidClientCredentialsException) as e:
            error = Error(e.getCode(), e.getMessage())
            bodyRS = BodyRS(False, error)

        response = Response(headerRS, bodyRS)
        return response;


        return response

    #Method to create user connection
    def loginUser(request):
        headerRS = HeaderRS()
        sessionRS = SessionRS(True)
        loginUserRQ = request.getBodyRQ()

        #Validate request inputs
        #bodyRS = new BodyRS()
        #bodyRS = Util.validateRQ(loginUserRQ);

        #if bodyRS is None:
        #    response.setBodyRS(bodyRS);
        #    response.setHeaderRS(new HeaderRS(request.getHeaderRQ().getToken()));
        #    return response;

        try:
            # Valid Connection Token
            connection = ConnectionService.validateConnection(request.getHeaderRQ().getToken());

            if loginUserRQ.getAuthenticationTypeId() == AuthenticationType.LOCAL:
                session = SessionService.createSessionByLocalAuth(connection, loginUserRQ.getUserName(), loginUserRQ.getToken())
            elif loginUserRQ.getAuthenticationTypeId() == AuthenticationType.GOOGLE:
                session = SessionService.createSessionByGoogleAuth(connection, loginUserRQ.getUserName(), loginUserRQ.getToken())
            elif loginUserRQ.getAuthenticationTypeId() == AuthenticationType.FACEBOOK:
                session = SessionService.createSessionByFacebookAuth(connection, loginUserRQ.getToken())
            elif loginUserRQ.getAuthenticationTypeId() == AuthenticationType.TOKEN:
                session = SessionService.createSessionByTokenAuth(connection, loginUserRQ.getUserName(), loginUserRQ.getToken())
            else:
                raise InvalidParamException("AuthenticationTypeId")

            sessionRS.setSession(session);
            #sessionRS.getRoles().addAll(userServiceLocal.getUserRoles(session.getUserId()));
            headerRS = HeaderMapper.mapToHeader(session);

        except (InvalidSecurityHashException, SecurityHashExpiredException, InvalidParamException, UserCredentialsExpiredException, NullClientConnectionException, InvalidClientConnectionException, ClientConnectionExpiredException, InternalErrorException, InvalidUserCredentialsException, NotFoundUserException, MaxUserSessionException, InvalidUserSessionException) as e:
            error = Error(e.getCode(), e.getMessage())
            sessionRS = SessionRS(False, error)

        response = Response(headerRS, sessionRS)
        return response

    def createUser(request):
        headerRS = HeaderRS()
        try:
            createUserRQ = request.getBodyRQ();

            SecurityHashService.validateHash(createUserRQ.getSecurityToken())

            user = UserService.createUser(createUserRQ.getUserName(), createUserRQ.getToken(), createUserRQ.getAuthenticationTypeId(), createUserRQ.getRoleId())
            headerRS = HeaderRS(request.getHeaderRQ().getToken())

            createUserRS = CreateUserRS(True)
            createUserRS.setUser(user)
        except  (InternalErrorException , NotFoundRoleException , UserAlreadyExistException) as e:
            error = Error(e.getCode(), e.getMessage())
            createUserRS = CreateUserRS(False, error)


        response = Response(headerRS, createUserRS)
        return response

    def createUserCreateToken(request):
        headerRS = HeaderRS()
        try:
            securityHash = SecurityHashService.createHash(SecurityHash.CREATE_USER_TOKEN)
            headerRS = HeaderRS(request.getHeaderRQ().getToken())
            createUserAccessTokenRS = CreateUserAccessTokenRS(True)
            createUserAccessTokenRS.setSecurityHash(securityHash)
        except  (InternalErrorException , NotFoundRoleException , UserAlreadyExistException) as e:
            error = Error(e.getCode(), e.getMessage())
            createUserAccessTokenRS = CreateUserTokenRS(False, error)

        response = Response(headerRS, createUserAccessTokenRS)
        return response

    def createUserAccessToken(request):
        headerRS = HeaderRS()
        try:
            createUserAccessTokenRQ = request.getBodyRQ();
            securityHash = UserService.createUserAccessToken(createUserAccessTokenRQ.getEmail())
            headerRS = HeaderRS(request.getHeaderRQ().getToken())
            createUserAccessTokenRS = CreateUserAccessTokenRS(True)
            createUserAccessTokenRS.setSecurityHash(securityHash)
        except  (InternalErrorException , NotFoundRoleException , UserAlreadyExistException) as e:
            error = Error(e.getCode(), e.getMessage())
            createUserAccessTokenRS = CreateUserTokenRS(False, error)


        response = Response(headerRS, createUserAccessTokenRS)
        return response



from flask import Flask

class Error(dict):

    def __init__(self, code, message, httpCode='200', type = None):
        self.__code = code
        self.__message = message
        self.__httpCode = httpCode
        self.__type = type
        dict.__init__(self, code=code, message=message, httpCode=httpCode)


    def getCode(self):
        return self.__code

    def getMessage(self):
        return self.__message



from cerberus.dtos.Connection import Connection

class ConnectionMapper():
    def mapToConnection(ksmConnection):
        return Connection(ksmConnection.getId(), ksmConnection.getClientId(), ksmConnection.getActive(), ksmConnection.getCreatedAt(), ksmConnection.getUpdatedAt())





import uuid
from datetime import datetime
from werkzeug.security import check_password_hash, generate_password_hash

from cerberus.mappers.UserMapper import UserMapper
from cerberus.entities.entities import KsmUserAuthenticationType, KsmRole, KsmUser, KsmRoleUser, Database
from cerberus.exceptions.exceptions import UserAlreadyExistException, NotFoundRoleException, InternalErrorException, NotFoundUserException
from cerberus.dtos.AuthenticationType import AuthenticationType
from cerberus.dtos.SecurityHash import SecurityHash
from cerberus.services.SecurityHashService import SecurityHashService
from cerberus.services.FirebaseService import FirebaseService

class UserService():

    def createUser(username, token, authenticationTypeId, roleId):

        ksmUser = None
        kuat = KsmUserAuthenticationType.query.filter(KsmUserAuthenticationType.username == username).filter(KsmUserAuthenticationType.authentication_type_id == authenticationTypeId).first()

        if not kuat is None:
            raise UserAlreadyExistException()

        kuat = KsmUserAuthenticationType.query.filter(KsmUserAuthenticationType.username == username).first()
        if not kuat is None:
            ksmUser = KsmUser.query.filter(KsmUser.id == kuat.getUserId()).first()

        ksmRole = KsmRole.query.filter(KsmRole.id == roleId).first()

        if ksmRole is None:
            raise NotFoundRoleException()

        if authenticationTypeId == AuthenticationType.GOOGLE:
            decoded_token = FirebaseService().getDecoded_token(token)

            res_uid = decoded_token['uid']
            if res_uid != username:
                print("Error:")
                print(res_uid)
                print(username)
                raise InvalidUserCredentialsException()

        try:
            createdAt = datetime.now()

            #Create Ksm User
            if ksmUser is None:
                ksmUser = KsmUser()
                ksmUser.setId(str(uuid.uuid4()))
                ksmUser.setCreatedAt(createdAt)
                ksmUser.setUpdatedAt(createdAt)
                Database.insert(ksmUser)

            #Create Ksm User Authentication Type
            UserService.createKsmUserAuthenticationType(ksmUser.getId(), username, token, authenticationTypeId)
            if authenticationTypeId == AuthenticationType.GOOGLE:
                email = decoded_token['email']
                UserService.createKsmUserAuthenticationType(ksmUser.getId(), email, token, AuthenticationType.TOKEN)

            #Add Role to User
            ksmRoleUser = KsmRoleUser()
            ksmRoleUser.setUserId(ksmUser.getId())
            ksmRoleUser.setRoleId(roleId)
            ksmRoleUser.setCreatedAt(createdAt)
            ksmRoleUser.setUpdatedAt(createdAt)
            Database.insert(ksmRoleUser)

        except Exception as ex:
            print(ex)
            raise InternalErrorException("Error creating user")

        return UserMapper.mapToUser(ksmUser)

    def createKsmUserAuthenticationType(userId, username, token, authenticationTypeId):
        createdAt = datetime.now()
        ksmUserAuthenticationType = KsmUserAuthenticationType()
        ksmUserAuthenticationType.setAuthenticationTypeId(authenticationTypeId)
        ksmUserAuthenticationType.setUserId(userId)
        ksmUserAuthenticationType.setUserName(username)
        ksmUserAuthenticationType.setCreatedAt(createdAt)
        ksmUserAuthenticationType.setUpdatedAt(createdAt)

        if authenticationTypeId == AuthenticationType.LOCAL:
            token = generate_password_hash(userId + token)

        ksmUserAuthenticationType.setToken(token)
        Database.insert(ksmUserAuthenticationType)

    def createUserAccessToken(email):

        ksmUserAuthenticationType = KsmUserAuthenticationType.query.filter(KsmUserAuthenticationType.username == email).first()
        if ksmUserAuthenticationType is None:
            raise NotFoundUserException()

        ksmUser = KsmUser.query.filter(KsmUser.id == ksmUserAuthenticationType.getUserId()).first()
        securityHash = SecurityHashService.createHash(SecurityHash.AUTH_TOKEN)

        createdAt = datetime.now()
        ksmUserAuthenticationType = KsmUserAuthenticationType.query.filter(KsmUserAuthenticationType.user_id == ksmUserAuthenticationType.getUserId()).filter(KsmUserAuthenticationType.authentication_type_id == AuthenticationType.TOKEN).first()
        if ksmUserAuthenticationType is None:
            ksmUserAuthenticationType = KsmUserAuthenticationType()
            ksmUserAuthenticationType.setAuthenticationTypeId(AuthenticationType.TOKEN)
            ksmUserAuthenticationType.setUserId(ksmUser.getId())
            ksmUserAuthenticationType.setUserName(username)
            ksmUserAuthenticationType.setCreatedAt(createdAt)
            ksmUserAuthenticationType.setUpdatedAt(createdAt)
            ksmUserAuthenticationType.setToken(generate_password_hash(securityHash.getToken()))
            Database.insert(ksmUserAuthenticationType)
        else:
            ksmUserAuthenticationType.setToken(generate_password_hash(securityHash.getToken()))
            ksmUserAuthenticationType.setUpdatedAt(createdAt)
            Database.update(ksmUserAuthenticationType)

        return securityHash
        
